@@ -12,6 +12,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <langinfo.h>
+#include <locale.h>
 #include <nl_types.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -276,6 +277,48 @@ char *xstpencpy(char *dest, char *end, const char *src, size_t n) {
 	}
 }
 
+const char *xstrerror(int errnum) {
+	int saved = errno;
+	const char *ret = NULL;
+	static thread_local char buf[256];
+
+#if __APPLE__
+	// No strerror_l() on macOS
+	if (strerror_r(errnum, buf, sizeof(buf)) == 0) {
+		ret = buf;
+	}
+#else
+#  if __NetBSD__
+	// NetBSD has no thread-specific locales
+	locale_t loc = LC_GLOBAL_LOCALE;
+#  else
+	locale_t loc = uselocale((locale_t)0);
+#  endif
+
+	locale_t copy = loc;
+	if (copy == LC_GLOBAL_LOCALE) {
+		copy = duplocale(copy);
+	}
+
+	if (copy != (locale_t)0) {
+		ret = strerror_l(errnum, copy);
+	}
+
+	if (loc == LC_GLOBAL_LOCALE) {
+		freelocale(copy);
+	}
+#endif
+
+	if (!ret) {
+		// Fallback for strerror_[lr]() or duplocale() failures
+		snprintf(buf, sizeof(buf), "Unknown error %d", errnum);
+		ret = buf;
+	}
+
+	errno = saved;
+	return ret;
+}
+
 void xstrmode(mode_t mode, char str[11]) {
 	strcpy(str, "----------");
 
@@ -651,7 +694,7 @@ static bool xiswprint(wchar_t c, enum wesc_flags flags) {
 /** Get the length of the longest printable prefix of a string. */
 static size_t printable_len(const char *str, size_t len, enum wesc_flags flags) {
 	static pthread_once_t once = PTHREAD_ONCE_INIT;
-	call_once(&once, char_cache_init);
+	invoke_once(&once, char_cache_init);
 
 	// Fast path: avoid multibyte checks
 	size_t i, word;
