@@ -318,8 +318,7 @@ static size_t bftw_child_nameoff(const struct bftw_file *parent) {
 
 /** Destroy a cache. */
 static void bftw_cache_destroy(struct bftw_cache *cache) {
-	bfs_assert(!cache->head);
-	bfs_assert(!cache->tail);
+	bfs_assert(LIST_EMPTY(cache));
 	bfs_assert(!cache->target);
 
 	varena_destroy(&cache->files);
@@ -347,9 +346,9 @@ static struct bftw_file *bftw_file_new(struct bftw_cache *cache, struct bftw_fil
 		file->nameoff = 0;
 	}
 
-	file->next = NULL;
-	file->to_read.next = NULL;
-	file->lru.prev = file->lru.next = NULL;
+	SLIST_ITEM_INIT(file);
+	SLIST_ITEM_INIT(file, to_read);
+	LIST_ITEM_INIT(file, lru);
 
 	file->refcount = 1;
 	file->pincount = 0;
@@ -397,6 +396,10 @@ static void bftw_file_free(struct bftw_cache *cache, struct bftw_file *file) {
  * Holds the current state of the bftw() traversal.
  */
 struct bftw_state {
+	/** The path(s) to start from. */
+	const char **paths;
+	/** The number of starting paths. */
+	size_t npaths;
 	/** bftw() callback. */
 	bftw_callback *callback;
 	/** bftw() callback data. */
@@ -419,6 +422,8 @@ struct bftw_state {
 	/** The number of I/O threads. */
 	size_t nthreads;
 
+	/** A batch of directories to open. */
+	struct bftw_list dir_batch;
 	/** The queue of directories to open. */
 	struct bftw_list to_open;
 	/** The queue of directories to read. */
@@ -426,13 +431,13 @@ struct bftw_state {
 	/** The queue of unpinned directories to unwrap. */
 	struct bftw_list to_close;
 
+	/** A batch of files to enqueue. */
+	struct bftw_list file_batch;
 	/** The queue of files to visit. */
 	struct bftw_list to_visit;
-	/** A batch of files to enqueue. */
-	struct bftw_list batch;
 
 	/** The current path. */
-	char *path;
+	dchar *path;
 	/** The current file. */
 	struct bftw_file *file;
 	/** The previous file. */
@@ -451,42 +456,84 @@ struct bftw_state {
 	struct BFTW ftwbuf;
 };
 
+/** Check if we have to buffer files before visiting them. */
+static bool bftw_must_buffer(const struct bftw_state *state) {
+	if (state->flags & BFTW_SORT) {
+		// Have to buffer the files to sort them
+		return true;
+	}
+
+	if (state->strategy == BFTW_DFS && state->nthreads == 0) {
+		// Without buffering, we would get a not-quite-depth-first
+		// ordering:
+		//
+		//     a
+		//     b
+		//     a/c
+		//     a/c/d
+		//     b/e
+		//     b/e/f
+		//
+		// This is okay for iterative deepening, since the caller only
+		// sees files at the target depth.  We also deem it okay for
+		// parallel searches, since the order is unpredictable anyway.
+		return true;
+	}
+
+	return false;
+}
+
 /** Initialize the bftw() state. */
 static int bftw_state_init(struct bftw_state *state, const struct bftw_args *args) {
+	state->paths = args->paths;
+	state->npaths = args->npaths;
 	state->callback = args->callback;
 	state->ptr = args->ptr;
 	state->flags = args->flags;
 	state->strategy = args->strategy;
 	state->mtab = args->mtab;
-
-	if ((state->flags & BFTW_SORT) || state->strategy == BFTW_DFS) {
-		state->flags |= BFTW_BUFFER;
-	}
-
 	state->error = 0;
 
-	if (args->nopenfd < 1) {
+	if (args->nopenfd < 2) {
 		errno = EMFILE;
 		return -1;
 	}
-	bftw_cache_init(&state->cache, args->nopenfd);
 
-	state->nthreads = args->nthreads;
-	if (state->nthreads > 0) {
-		state->ioq = ioq_create(4096, state->nthreads);
+	size_t nopenfd = args->nopenfd;
+	size_t qdepth = 4096;
+	size_t nthreads = args->nthreads;
+
+#if BFS_USE_LIBURING
+	// io_uring uses one fd per ring, ioq uses one ring per thread
+	if (nthreads >= nopenfd - 1) {
+		nthreads = nopenfd - 2;
+	}
+	nopenfd -= nthreads;
+#endif
+
+	bftw_cache_init(&state->cache, nopenfd);
+
+	if (nthreads > 0) {
+		state->ioq = ioq_create(qdepth, nthreads);
 		if (!state->ioq) {
 			return -1;
 		}
 	} else {
 		state->ioq = NULL;
 	}
+	state->nthreads = nthreads;
 
+	if (bftw_must_buffer(state)) {
+		state->flags |= BFTW_BUFFER;
+	}
+
+	SLIST_INIT(&state->dir_batch);
 	SLIST_INIT(&state->to_open);
 	SLIST_INIT(&state->to_read);
 	SLIST_INIT(&state->to_close);
 
+	SLIST_INIT(&state->file_batch);
 	SLIST_INIT(&state->to_visit);
-	SLIST_INIT(&state->batch);
 
 	state->path = NULL;
 	state->file = NULL;
@@ -499,6 +546,17 @@ static int bftw_state_init(struct bftw_state *state, const struct bftw_args *arg
 	return 0;
 }
 
+/** Unpin a directory, and possibly queue it for unwrapping. */
+static void bftw_unpin_dir(struct bftw_state *state, struct bftw_file *file, bool force) {
+	bftw_cache_unpin(&state->cache, file);
+
+	if (file->dir && (force || file->pincount == 0)) {
+		if (!SLIST_ATTACHED(&state->to_close, file)) {
+			SLIST_APPEND(&state->to_close, file);
+		}
+	}
+}
+
 /** Pop a response from the I/O queue. */
 static int bftw_ioq_pop(struct bftw_state *state, bool block) {
 	struct ioq *ioq = state->ioq;
@@ -535,10 +593,7 @@ static int bftw_ioq_pop(struct bftw_state *state, bool block) {
 		++cache->capacity;
 		parent = file->parent;
 		if (parent) {
-			bftw_cache_unpin(cache, parent);
-			if (parent->pincount == 0 && parent->dir) {
-				SLIST_APPEND(&state->to_close, parent);
-			}
+			bftw_unpin_dir(state, parent, false);
 		}
 
 		dir = ent->opendir.dir;
@@ -756,8 +811,12 @@ static int bftw_unwrapdir(struct bftw_state *state, struct bftw_file *file) {
 		return bftw_close(state, file);
 	}
 
-	if (bftw_cache_reserve(state) != 0) {
-		return -1;
+	// Make room for dup()
+	bftw_cache_pin(cache, file);
+	int ret = bftw_cache_reserve(state);
+	bftw_cache_unpin(cache, file);
+	if (ret != 0) {
+		return ret;
 	}
 
 	int fd = dup_cloexec(file->fd);
@@ -815,11 +874,32 @@ fail:
 	return -1;
 }
 
+/** Open a batch of directories asynchronously. */
+static void bftw_ioq_opendirs(struct bftw_state *state, struct bftw_list *queue) {
+	for_slist (struct bftw_file, dir, queue) {
+		if (bftw_ioq_opendir(state, dir) != 0) {
+			break;
+		}
+		SLIST_POP(queue);
+	}
+}
+
 /** Push a directory onto the queue. */
 static void bftw_push_dir(struct bftw_state *state, struct bftw_file *file) {
 	bfs_assert(file->type == BFS_DIR);
 
-	SLIST_APPEND(&state->to_open, file);
+	struct bftw_list *queue;
+	if (state->strategy == BFTW_BFS || (state->flags & BFTW_BUFFER)) {
+		// In breadth-first mode, or if we're already buffering files,
+		// we can push directly to the to_open queue
+		queue = &state->to_open;
+	} else {
+		// For a depth-first, unbuffered search, add directories to a
+		// batch, then push the patch to the front of the queue
+		queue = &state->dir_batch;
+	}
+
+	SLIST_APPEND(queue, file);
 
 	if (state->flags & BFTW_SORT) {
 		// When sorting, directories are kept in order on the to_read
@@ -827,13 +907,7 @@ static void bftw_push_dir(struct bftw_state *state, struct bftw_file *file) {
 		SLIST_APPEND(&state->to_read, file, to_read);
 	}
 
-	while (state->to_open.head) {
-		if (bftw_ioq_opendir(state, state->to_open.head) == 0) {
-			SLIST_POP(&state->to_open);
-		} else {
-			break;
-		}
-	}
+	bftw_ioq_opendirs(state, queue);
 }
 
 /** Pop a directory to read from the queue. */
@@ -841,17 +915,17 @@ static bool bftw_pop_dir(struct bftw_state *state) {
 	bfs_assert(!state->file);
 
 	struct bftw_cache *cache = &state->cache;
-	bool have_files = state->to_visit.head;
+	bool have_files = !SLIST_EMPTY(&state->to_visit);
 
 	if (state->flags & BFTW_SORT) {
 		// Keep strict breadth-first order when sorting
-		if (state->strategy != BFTW_DFS && have_files) {
+		if (state->strategy == BFTW_BFS && have_files) {
 			return false;
 		}
 	} else {
-		while (!state->to_read.head) {
+		while (SLIST_EMPTY(&state->to_read)) {
 			// Block if we have no other files/dirs to visit, or no room in the cache
-			bool have_dirs = state->to_open.head;
+			bool have_dirs = !SLIST_EMPTY(&state->to_open);
 			bool have_room = cache->capacity > 0 && cache->dirlimit > 0;
 			bool block = !(have_dirs || have_files) || !have_room;
 
@@ -1223,8 +1297,7 @@ static int bftw_gc(struct bftw_state *state, enum bftw_gc_flags flags) {
 
 	struct bftw_file *file = state->file;
 	if (file && file->dir) {
-		bftw_cache_unpin(&state->cache, file);
-		SLIST_APPEND(&state->to_close, file);
+		bftw_unpin_dir(state, file, true);
 	}
 	state->dir = NULL;
 	state->de = NULL;
@@ -1297,7 +1370,7 @@ static void bftw_list_sort(struct bftw_list *list) {
 	bftw_list_sort(&right);
 
 	// Merge
-	while (left.head && right.head) {
+	while (!SLIST_EMPTY(&left) && !SLIST_EMPTY(&right)) {
 		struct bftw_file *lf = left.head;
 		struct bftw_file *rf = right.head;
 
@@ -1316,13 +1389,18 @@ static void bftw_list_sort(struct bftw_list *list) {
 /** Finish adding a batch of files. */
 static void bftw_batch_finish(struct bftw_state *state) {
 	if (state->flags & BFTW_SORT) {
-		bftw_list_sort(&state->batch);
+		bftw_list_sort(&state->file_batch);
 	}
 
 	if (state->strategy != BFTW_BFS) {
-		SLIST_EXTEND(&state->batch, &state->to_visit);
+		SLIST_EXTEND(&state->dir_batch, &state->to_open);
+		SLIST_EXTEND(&state->file_batch, &state->to_visit);
 	}
-	SLIST_EXTEND(&state->to_visit, &state->batch);
+
+	SLIST_EXTEND(&state->to_open, &state->dir_batch);
+	SLIST_EXTEND(&state->to_visit, &state->file_batch);
+
+	bftw_ioq_opendirs(state, &state->to_open);
 }
 
 /** Close the current directory. */
@@ -1364,7 +1442,7 @@ static int bftw_visit(struct bftw_state *state, const char *name) {
 			file->type = state->de->type;
 		}
 
-		SLIST_APPEND(&state->batch, file);
+		SLIST_APPEND(&state->file_batch, file);
 		return 0;
 	}
 
@@ -1412,7 +1490,8 @@ static int bftw_state_destroy(struct bftw_state *state) {
 		state->ioq = NULL;
 	}
 
-	SLIST_EXTEND(&state->to_visit, &state->batch);
+	SLIST_EXTEND(&state->to_open, &state->dir_batch);
+	SLIST_EXTEND(&state->to_visit, &state->file_batch);
 	do {
 		bftw_gc(state, BFTW_VISIT_NONE);
 	} while (bftw_pop_dir(state) || bftw_pop_file(state));
@@ -1425,46 +1504,53 @@ static int bftw_state_destroy(struct bftw_state *state) {
 	return state->error ? -1 : 0;
 }
 
+/**
+ * Shared implementation for all search strategies.
+ */
+static int bftw_impl(struct bftw_state *state) {
+	for (size_t i = 0; i < state->npaths; ++i) {
+		if (bftw_visit(state, state->paths[i]) != 0) {
+			return -1;
+		}
+	}
+	bftw_batch_finish(state);
+
+	while (true) {
+		while (bftw_pop_dir(state)) {
+			if (bftw_opendir(state) != 0) {
+				return -1;
+			}
+			while (bftw_readdir(state) > 0) {
+				if (bftw_visit(state, state->de->name) != 0) {
+					return -1;
+				}
+			}
+			if (bftw_closedir(state) != 0) {
+				return -1;
+			}
+		}
+
+		if (!bftw_pop_file(state)) {
+			break;
+		}
+		if (bftw_visit(state, NULL) != 0) {
+			break;
+		}
+	}
+
+	return 0;
+}
+
 /**
  * bftw() implementation for simple breadth-/depth-first search.
  */
-static int bftw_impl(const struct bftw_args *args) {
+static int bftw_walk(const struct bftw_args *args) {
 	struct bftw_state state;
 	if (bftw_state_init(&state, args) != 0) {
 		return -1;
 	}
 
-	for (size_t i = 0; i < args->npaths; ++i) {
-		if (bftw_visit(&state, args->paths[i]) != 0) {
-			goto done;
-		}
-	}
-	bftw_batch_finish(&state);
-
-	while (true) {
-		while (bftw_pop_dir(&state)) {
-			if (bftw_opendir(&state) != 0) {
-				goto done;
-			}
-			while (bftw_readdir(&state) > 0) {
-				if (bftw_visit(&state, state.de->name) != 0) {
-					goto done;
-				}
-			}
-			if (bftw_closedir(&state) != 0) {
-				goto done;
-			}
-		}
-
-		if (!bftw_pop_file(&state)) {
-			break;
-		}
-		if (bftw_visit(&state, NULL) != 0) {
-			break;
-		}
-	}
-
-done:
+	bftw_impl(&state);
 	return bftw_state_destroy(&state);
 }
 
@@ -1472,6 +1558,8 @@ done:
  * Iterative deepening search state.
  */
 struct bftw_ids_state {
+	/** Nested walk state. */
+	struct bftw_state nested;
 	/** The wrapped callback. */
 	bftw_callback *delegate;
 	/** The wrapped callback arguments. */
@@ -1486,12 +1574,8 @@ struct bftw_ids_state {
 	size_t max_depth;
 	/** The set of pruned paths. */
 	struct trie pruned;
-	/** An error code to report. */
-	int error;
 	/** Whether the bottom has been found. */
 	bool bottom;
-	/** Whether to quit the search. */
-	bool quit;
 };
 
 /** Iterative deepening callback function. */
@@ -1535,17 +1619,17 @@ static enum bftw_action bftw_ids_callback(const struct BFTW *ftwbuf, void *ptr)
 			ret = BFTW_PRUNE;
 		}
 		break;
+
 	case BFTW_PRUNE:
 		if (ftwbuf->type == BFS_DIR) {
 			if (!trie_insert_str(&state->pruned, ftwbuf->path)) {
-				state->error = errno;
-				state->quit = true;
+				state->nested.error = errno;
 				ret = BFTW_STOP;
 			}
 		}
 		break;
+
 	case BFTW_STOP:
-		state->quit = true;
 		break;
 	}
 
@@ -1553,7 +1637,7 @@ static enum bftw_action bftw_ids_callback(const struct BFTW *ftwbuf, void *ptr)
 }
 
 /** Initialize iterative deepening state. */
-static void bftw_ids_init(const struct bftw_args *args, struct bftw_ids_state *state, struct bftw_args *ids_args) {
+static int bftw_ids_init(struct bftw_ids_state *state, const struct bftw_args *args) {
 	state->delegate = args->callback;
 	state->ptr = args->ptr;
 	state->visit = BFTW_PRE;
@@ -1561,30 +1645,19 @@ static void bftw_ids_init(const struct bftw_args *args, struct bftw_ids_state *s
 	state->min_depth = 0;
 	state->max_depth = 1;
 	trie_init(&state->pruned);
-	state->error = 0;
 	state->bottom = false;
-	state->quit = false;
 
-	*ids_args = *args;
-	ids_args->callback = bftw_ids_callback;
-	ids_args->ptr = state;
-	ids_args->flags &= ~BFTW_POST_ORDER;
+	struct bftw_args ids_args = *args;
+	ids_args.callback = bftw_ids_callback;
+	ids_args.ptr = state;
+	ids_args.flags &= ~BFTW_POST_ORDER;
+	return bftw_state_init(&state->nested, &ids_args);
 }
 
 /** Finish an iterative deepening search. */
-static int bftw_ids_finish(struct bftw_ids_state *state) {
-	int ret = 0;
-
-	if (state->error) {
-		ret = -1;
-	} else {
-		state->error = errno;
-	}
-
+static int bftw_ids_destroy(struct bftw_ids_state *state) {
 	trie_destroy(&state->pruned);
-
-	errno = state->error;
-	return ret;
+	return bftw_state_destroy(&state->nested);
 }
 
 /**
@@ -1592,15 +1665,15 @@ static int bftw_ids_finish(struct bftw_ids_state *state) {
  */
 static int bftw_ids(const struct bftw_args *args) {
 	struct bftw_ids_state state;
-	struct bftw_args ids_args;
-	bftw_ids_init(args, &state, &ids_args);
+	if (bftw_ids_init(&state, args) != 0) {
+		return -1;
+	}
 
-	while (!state.quit && !state.bottom) {
+	while (!state.bottom) {
 		state.bottom = true;
 
-		if (bftw_impl(&ids_args) != 0) {
-			state.error = errno;
-			state.quit = true;
+		if (bftw_impl(&state.nested) != 0) {
+			goto done;
 		}
 
 		++state.min_depth;
@@ -1611,18 +1684,18 @@ static int bftw_ids(const struct bftw_args *args) {
 		state.visit = BFTW_POST;
 		state.force_visit = true;
 
-		while (!state.quit && state.min_depth > 0) {
+		while (state.min_depth > 0) {
 			--state.max_depth;
 			--state.min_depth;
 
-			if (bftw_impl(&ids_args) != 0) {
-				state.error = errno;
-				state.quit = true;
+			if (bftw_impl(&state.nested) != 0) {
+				goto done;
 			}
 		}
 	}
 
-	return bftw_ids_finish(&state);
+done:
+	return bftw_ids_destroy(&state);
 }
 
 /**
@@ -1630,39 +1703,38 @@ static int bftw_ids(const struct bftw_args *args) {
  */
 static int bftw_eds(const struct bftw_args *args) {
 	struct bftw_ids_state state;
-	struct bftw_args ids_args;
-	bftw_ids_init(args, &state, &ids_args);
+	if (bftw_ids_init(&state, args) != 0) {
+		return -1;
+	}
 
-	while (!state.quit && !state.bottom) {
+	while (!state.bottom) {
 		state.bottom = true;
 
-		if (bftw_impl(&ids_args) != 0) {
-			state.error = errno;
-			state.quit = true;
+		if (bftw_impl(&state.nested) != 0) {
+			goto done;
 		}
 
 		state.min_depth = state.max_depth;
 		state.max_depth *= 2;
 	}
 
-	if (!state.quit && (args->flags & BFTW_POST_ORDER)) {
+	if (args->flags & BFTW_POST_ORDER) {
 		state.visit = BFTW_POST;
 		state.min_depth = 0;
-		ids_args.flags |= BFTW_POST_ORDER;
+		state.nested.flags |= BFTW_POST_ORDER;
 
-		if (bftw_impl(&ids_args) != 0) {
-			state.error = errno;
-		}
+		bftw_impl(&state.nested);
 	}
 
-	return bftw_ids_finish(&state);
+done:
+	return bftw_ids_destroy(&state);
 }
 
 int bftw(const struct bftw_args *args) {
 	switch (args->strategy) {
 	case BFTW_BFS:
 	case BFTW_DFS:
-		return bftw_impl(args);
+		return bftw_walk(args);
 	case BFTW_IDS:
 		return bftw_ids(args);
 	case BFTW_EDS:
