@@ -328,7 +328,7 @@ fail:
 static int build_iext_trie(struct colors *colors) {
 	trie_clear(&colors->iext_trie);
 
-	TRIE_FOR_EACH(&colors->ext_trie, leaf) {
+	for_trie (leaf, &colors->ext_trie) {
 		size_t len = leaf->length - 1;
 		if (colors->ext_len < len) {
 			colors->ext_len = len;
@@ -558,8 +558,8 @@ static int unescape(char **str, const char *value, char end, const char **next)
 /** Parse the GNU $LS_COLORS format. */
 static int parse_gnu_ls_colors(struct colors *colors, const char *ls_colors) {
 	int ret = -1;
-	char *key = NULL;
-	char *value = NULL;
+	dchar *key = NULL;
+	dchar *value = NULL;
 
 	for (const char *chunk = ls_colors, *next; chunk; chunk = next) {
 		if (chunk[0] == '*') {
@@ -943,7 +943,7 @@ static ssize_t first_broken_offset(const char *path, const struct BFTW *ftwbuf,
 		goto out;
 	}
 
-	char *at_path;
+	dchar *at_path;
 	int at_fd;
 	if (path == ftwbuf->path) {
 		if (ftwbuf->depth == 0) {
@@ -1143,11 +1143,11 @@ static int print_expr(CFILE *cfile, const struct bfs_expr *expr, bool verbose) {
 	if (verbose) {
 		double rate = 0.0, time = 0.0;
 		if (expr->evaluations) {
-			rate = 100.0*expr->successes/expr->evaluations;
-			time = (1.0e9*expr->elapsed.tv_sec + expr->elapsed.tv_nsec)/expr->evaluations;
+			rate = 100.0 * expr->successes / expr->evaluations;
+			time = (1.0e9 * expr->elapsed.tv_sec + expr->elapsed.tv_nsec) / expr->evaluations;
 		}
 		if (cbuff(cfile, " [${ylw}%zu${rs}/${ylw}%zu${rs}=${ylw}%g%%${rs}; ${ylw}%gns${rs}]",
-		          expr->successes, expr->evaluations, rate, time)) {
+			    expr->successes, expr->evaluations, rate, time)) {
 			return -1;
 		}
 	}
@@ -1239,7 +1239,7 @@ static int cvbuff(CFILE *cfile, const char *format, va_list args) {
 				break;
 
 			case 'm':
-				if (dstrcat(&cfile->buffer, strerror(error)) != 0) {
+				if (dstrcat(&cfile->buffer, xstrerror(error)) != 0) {
 					return -1;
 				}
 				break;
