@@ -1,5 +1,5 @@
-skip_unless invoke_bfs scratch -quit -xattr
-skip_unless make_xattrs
+invoke_bfs scratch -quit -xattr || skip
+make_xattrs || skip
 
 case "$UNAME" in
     Darwin|FreeBSD)
