@@ -2,7 +2,7 @@
 
 clean_scratch
 mkdir scratch/foo
-$TOUCH scratch/foo/{1..256}
+"$XTOUCH" scratch/foo/{1..256}
 
 invoke_bfs scratch/foo -delete
 bfs_diff scratch
