@@ -1,5 +1,5 @@
 # fnmatch() is broken on macOS
-skip_if test "$UNAME" = "Darwin"
+test "$UNAME" = "Darwin" && skip
 
 # An unclosed [ should be matched literally
 bfs_diff weirdnames -name '['
