@@ -19,7 +19,11 @@ export UBSAN_OPTIONS="$SAN_OPTIONS"
 export LS_COLORS=""
 unset BFS_COLORS
 
-if [ -t 1 ]; then
+function color_fd() {
+    [ -z "${NO_COLOR:-}" ] && [ -t "$1" ]
+}
+
+if color_fd 1; then
     BLD=$'\033[01m'
     RED=$'\033[01;31m'
     GRN=$'\033[01;32m'
@@ -389,21 +393,18 @@ function make_deep() {
     name="${name}${name}${name}${name}"
     name="${name:0:255}"
 
-    # 4 * 256 - 1 == 1023
-    local names="$name/$name/$name/$name"
-
     for i in {0..9} A B C D E F; do
+        "$XTOUCH" -p "$1/$i/$name"
+
         (
-            mkdir "$1/$i"
             cd "$1/$i"
 
-            # 4 * 1024 == 4096 == PATH_MAX
-            for _ in {1..4}; do
-                mkdir -p "$names"
-                cd "$names"
+            # 8 * 512 == 4096 >= PATH_MAX
+            for _ in {1..8}; do
+                mv "$name" ..
+                mkdir -p "$name/$name"
+                mv "../$name" "$name/$name/"
             done
-
-            "$XTOUCH" "$name"
         )
     done
 }
@@ -446,7 +447,7 @@ fi
 
 function bfs_verbose() {
     if [ "$VERBOSE_COMMANDS" ]; then
-        if [ -t 3 ]; then
+        if color_fd 3; then
             printf "${GRN}%q${RST} " "${BFS[@]}" >&3
 
             local expr_started=
@@ -487,6 +488,26 @@ function invoke_bfs() {
     fi
 }
 
+if command -v unbuffer &>/dev/null; then
+    UNBUFFER=unbuffer
+elif command -v expect_unbuffer &>/dev/null; then
+    UNBUFFER=expect_unbuffer
+fi
+
+function bfs_pty() {
+    test -n "${UNBUFFER:-}" || skip
+
+    bfs_verbose "$@"
+    "$UNBUFFER" bash -c 'stty cols 80 rows 24 && "$@"' bash "${BFS[@]}" "$@"
+    local status="$?"
+
+    if ((status > 125)); then
+        exit "$status"
+    else
+        return "$status"
+    fi
+}
+
 function check_exit() {
     local expected="$1"
     local actual="0"
@@ -496,7 +517,7 @@ function check_exit() {
 }
 
 # Detect colored diff support
-if [ -t 2 ] && diff --color=always /dev/null /dev/null 2>/dev/null; then
+if color_fd 2 && diff --color=always /dev/null /dev/null 2>/dev/null; then
     DIFF="diff --color=always"
 else
     DIFF="diff"
@@ -637,7 +658,7 @@ function update_eol() {
 
 if [ "$VERBOSE_TESTS" ]; then
     BOL=''
-elif [ -t 1 ]; then
+elif color_fd 1; then
     BOL='\r\033[K'
 
     # Workaround for bash 4: checkwinsize is off by default.  We can turn it on,
@@ -654,12 +675,14 @@ passed=0
 failed=0
 skipped=0
 
+if color_fd 1 || [ "$VERBOSE_TESTS" ]; then
+    TEST_FMT="${BOL}${YLW}%s${RST}${EOL}"
+else
+    TEST_FMT="."
+fi
+
 for TEST in "${TEST_CASES[@]}"; do
-    if [[ -t 1 || "$VERBOSE_TESTS" ]]; then
-        printf "${BOL}${YLW}%s${RST}${EOL}" "$TEST"
-    else
-        printf "."
-    fi
+    printf "$TEST_FMT" "$TEST"
 
     OUT="$TMP/$TEST.out"
     mkdir -p "${OUT%/*}"
